package ru.dbotthepony.mc.otm.capability.matter

import net.minecraftforge.common.capabilities.ICapabilityProvider
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.core.Decimal
import ru.dbotthepony.mc.otm.core.RGBAColor
import ru.dbotthepony.mc.otm.core.orNull
import kotlin.math.roundToInt

interface IMatterHandler {
	/**
	 * If this is false, then [storedMatter] will throw [UnsupportedOperationException] when trying to set it
	 */
	val canSetMatterLevel: Boolean get() = true

	/**
	 * Implementations are free to throw [UnsupportedOperationException] if setting battery level is not supported
	 * due to technical complications (in this case, [canSetMatterLevel] MUST be false)
	 *
	 * @throws [UnsupportedOperationException]
	 */
	var storedMatter: Decimal
	val maxStoredMatter: Decimal

	/**
	 * Empties matter stored of this matter storage
	 *
	 * @throws [UnsupportedOperationException]
	 * @see storedMatter
	 */
	fun emptyMatter() {
		storedMatter = Decimal.ZERO
	}

	/**
	 * Fully fills matter stored in this matter storage
	 *
	 * @throws [UnsupportedOperationException]
	 * @see storedMatter
	 */
	fun fillMatter() {
		storedMatter = maxStoredMatter
	}

	fun receiveMatterOuter(howMuch: Decimal, simulate: Boolean): Decimal
	fun receiveMatterInner(howMuch: Decimal, simulate: Boolean): Decimal
	fun extractMatterOuter(howMuch: Decimal, simulate: Boolean): Decimal
	fun extractMatterInner(howMuch: Decimal, simulate: Boolean): Decimal

	val direction: MatterDirection
	val missingMatter: Decimal
		get() = maxStoredMatter.minus(storedMatter).moreThanZero()

	val allowsExtract: Boolean
		get() = direction != MatterDirection.RECEIVE

	val allowsReceive: Boolean
		get() = direction != MatterDirection.EXTRACT
}

fun IMatterHandler.getBarWidth(): Int {
	return ((storedMatter / maxStoredMatter).toFloat().coerceAtLeast(0f).coerceAtMost(1f) * 13f).roundToInt()
}

fun IMatterHandler.getBarColor(): Int {
	return RGBAColor.LOW_MATTER.linearInterpolation((storedMatter / maxStoredMatter).toFloat(), RGBAColor.FULL_MATTER).toInt()
}

val ICapabilityProvider.matter: IMatterHandler? get() = getCapability(MatteryCapability.MATTER).orNull()
